# Scraft

React library for JSON configured apps.

## Features
- Exposes a single React Hook named `useBuilder`
- Can be used with your existing React components
- JSON configuration supports string interpolation & function composition
- Page routing and layout support

## Installing
Using npm:

```bash
$ npm install --save scraft
```

Using yarn:

```bash
$ yarn add scraft
```

## Usage

`App.js`
```js 
import React from 'react';
import { useBuilder } from 'scraft';
import { AppBar, Tabs, ColorBlock } from './components';

const config = readFile('./config.json');
const components = { AppBar, Tabs, ColorBlock };
const utils = { log: message => console.log(message) };

const App = props => {
	const BuiltApp = useBuilder(config, components, utils);
	return <BuiltApp />;
};
```

`config.json`
```json
{
    "app": {
        "containerIds": ["appBar"],
        "routes": [
            {
                "path": "/",
                "exact": true,
                "pageId": "rootPage"
            }
        ]
    },
    "pages": [
        {
            "id": "rootPage",
            "containerIds": ["pageTabs"],
            "widgetIds": ["redBlock"],
            "widgetLayouts": {
                "lg": [
                    {
                        "i": "redBlock",
                        "x": 0,
                        "y": 0,
                        "w": 12,
                        "h": 4
                    }
                ]
            }
        }
    ],
    "containers": [
        {
            "id": "appBar",
            "component": {
                "type": "AppBar"
            }
        },
        {
            "id": "pageTabs",
            "component": {
                "type": "Tabs"
            }
        }
    ],
    "widgets": [
        {
            "id": "redBlock",
            "handlers": {
                "notifyClick": {
                    "function": "${utils.log}",
                    "parameters": ["Red block clicked!"]
                }
            },
            "component": {
                "type": "ColorBlock",
                "props": {
                    "background": "red",
                    "onClick": "${handlers.notifyClick}"
                }
            }
        }
    ]
}
```

## Configuration API

### App
Configures the initial application level state, and containers. These will persist throughout the entire lifetime of the application. Routing within the application is also configured here.

##### Prop Types
```js
{
  initState: PropTypes.object,
  containerIds: PropTypes.arrayOf(PropTypes.string),
  routes: PropTypes.arrayOf(
    PropTypes.shape({
      path: PropTypes.string.isRequired,
      pageId: PropTypes.string.isRequired
    })
  ).isRequired
}
```

### Pages
Configures initial page level state, containers, and widgets. These will persist for as long as the page is mounted. Configuration for widget layout & grid is also available. The widget grid has 12 columns, and the following breakpoints:

- lg: >= 1200px
- md: >= 992px 
- sm: >= 768px
- xs: < 768px

##### Prop Types
```js
{
  id: PropTypes.string.isRequired,
  initState: PropTypes.object,
  initLayoutMode: PropTypes.bool,
  containerIds: PropTypes.arrayOf(PropTypes.string),
  widgetIds: PropTypes.arrayOf(PropTypes.string),
  widgetLayouts: PropTypes.objectOf(
    PropTypes.arrayOf(
      PropTypes.shape({
        i: PropTypes.string.isRequired,
        x: PropTypes.number.isRequired,
        y: PropTypes.number.isRequired,
        w: PropTypes.number.isRequired,
        h: PropTypes.number.isRequired
      })
    )
  ).isRequired,
  widgetGrid: PropTypes.shape({
    rowHeight: PropTypes.number,
    margin: PropTypes.arrayOf(PropTypes.number),
    padding: PropTypes.arrayOf(PropTypes.number),
    cols: PropTypes.objectOf(
      PropTypes.shape({
        lg: PropTypes.number,
        md: PropTypes.number,
        sm: PropTypes.number,
        xs: PropTypes.number
      })
    )
  })
}
```

### Containers & Widgets
Containers and widgets share the same wrapper component which allows for the interpolation of their props.
Each of these also allows for the composition of functions via the `handlers` property. Container components all receive the widgets as children so it's important to pass them down the tree like the following:

```js
const Container = props => {
  return <div>{props.children}</div>;
};
```

##### Prop Types

```js
[
  ...
  {
    id: PropTypes.string.isRequired,
    initState: PropTypes.object,
    handlers: PropTypes.objectOf(
      PropTypes.arrayOf(
          PropTypes.shape({
              "function": PropTypes.string.isRequired,
              "parameters": PropTypes.array.isRequired
          })
      )
    ),
    onMount: PropTypes.string,
    onUnmount: PropTypes.string,
    component: PropTypes.shape({
      type: PropTypes.string.isRequired,
      props: PropTypes.object
    }).isRequired
  }
]
```
